﻿/* 
    Copyright (c) 2010 Microsoft Corporation.  All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
*/
using System;
using System.IO;
using System.Threading;
using System.Windows;
using System.Windows.Media.Imaging;
using System.Windows.Threading;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;

namespace P27_2
{
    public partial class MainPage : PhoneApplicationPage
    {
        // Объект, представляющий физический микрофон на устройстве
        private Microphone microphone = Microphone.Default;  
        //Динамический буфер для получения аудиоданных с микрофона
        private byte[] buffer;               
        //Хранение аудиоданных для последующего воспроизведения           
        private MemoryStream stream = new MemoryStream();   
        //Используется для воспроизведения звука
        private SoundEffectInstance soundInstance;          
        //Флаг для отслеживания состояния проигрывания звука
        private bool soundIsPlaying = false;

        // Изображения, сигнализирующие о состоянии программы
        private BitmapImage blankImage;
        private BitmapImage microphoneImage;
        private BitmapImage speakerImage;

        /// <summary>
        /// Конструктор
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            // Таймер для организации игрового цикла XNA (обект Microphone взят 
            // из XNA Framework). Мы так же используем этот таймер для отслеживания
            // состояния воспроизведения аудио, что позволяем нам обновлять
            // состояние пользовательского интерфейса
            DispatcherTimer dt = new DispatcherTimer();
            dt.Interval = TimeSpan.FromMilliseconds(33);
            dt.Tick += new EventHandler(dt_Tick);
            dt.Start();

            //Обработчик событя для получения аудиоданных, когда буфер заполнится 
            microphone.BufferReady += new EventHandler<EventArgs>(microphone_BufferReady);
            //Подготовка ссылок на изображения
            blankImage = new BitmapImage(new Uri("Images/blank.png", UriKind.RelativeOrAbsolute));
            microphoneImage = new BitmapImage(new Uri("Images/microphone.png", UriKind.RelativeOrAbsolute));
            speakerImage = new BitmapImage(new Uri("Images/speaker.png", UriKind.RelativeOrAbsolute));
        }

        /// <summary>
        /// Вызвает XNA FrameworkDispatcher и проверяет, проигрывается ли звук.
        /// Если проигрывание завершено, обновляет состояние пользовательского интерфейса.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void dt_Tick(object sender, EventArgs e)
        {
            //Организуем игровой цикл XNA
            try { FrameworkDispatcher.Update(); }
            catch { }
            //Если звук проигрывается
            if (true == soundIsPlaying)
            {
                //Если проигрывание завершено
                if (soundInstance.State != SoundState.Playing)
                {
                    // Проигрывание завершено
                    soundIsPlaying = false;

                    // Обновляет пользовательский интерфейс
                    // того, чтобы отразить, что прекращено проигрывание звука
                    SetButtonStates(true, true, false);
                    UserHelp.Text = "Нажмите кнопку Проиграть или Запись";
                    StatusImage.Source = blankImage;
                }
            }
        }

        /// <summary>
        /// Обработчик события Microphone.BufferReady.
        /// Получает аудиоданные с микрофона, сохраняет их в буфер,
        /// затем записывает содержимое буфера в поток для последующего воспроизведения.
        /// Любые действия в этом обработчике должны быть как можно более быстрыми!
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void microphone_BufferReady(object sender, EventArgs e)
        {
            // Получаем аудиоданные
            microphone.GetData(buffer);

            // Сохраняем данные в поток
            stream.Write(buffer, 0, buffer.Length);
        }

        /// <summary>
        /// Обрабатывает событие Click для кнопки record (запись).
        /// Устанавливает микрофон и буферы данных для получения аудиоданных,
        /// затем запускает микрофон. Кроме того, обновляет пользовательский интерфейс.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void recordButton_Click(object sender, EventArgs e)
        {
            // Получение аудиоданных участками по 1/2 секунды
            microphone.BufferDuration = TimeSpan.FromMilliseconds(500);

            // Выделяет память для хранения аудиоданных
            buffer = new byte[microphone.GetSampleSizeInBytes(microphone.BufferDuration)];

            // Сбрасывает поток в 0 для того случая, если он уже содержит
            //какие-либо данные
            stream.SetLength(0);

            // Начало записи
            microphone.Start();
            
            // Обновление пользовательского интерфейса
            SetButtonStates(false, false, true);
            UserHelp.Text = "Идёт запись...";
            StatusImage.Source = microphoneImage;
        }

        /// <summary>
        /// Обрабатывает событие Click для кнопки stop (стоп).
        /// Останавливает запись звука с микрофона и обновляет пользовательский интерфейс.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            //Если запись идёт
            if (microphone.State == MicrophoneState.Started)
            {
                // В режиме записи пользователь нажимает на кнопку  
                // stop для того, чтобы прекратить запись
                microphone.Stop();
            }
            // Если звук воспроизводится
            else if (soundInstance.State == SoundState.Playing)
            {
                // В режиме воспроизведения пользоатель нажимает на кнопку 
                // stop для того, чтобы остановить воспроизведение
                soundInstance.Stop();
            }
            // Обновляет пользовательский интерфейс
            SetButtonStates(true, true, false);
            UserHelp.Text = "Готово";
            StatusImage.Source = blankImage;
        }

        /// <summary>
        /// Обрабатывает событие Click для кнопки play (воспроизведение).
        /// Проигрывает звук, записанный в прошлом сеансе записи с микрофона
        /// и обновляет пользовательский интерфейс.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void playButton_Click(object sender, EventArgs e)
        {
            if (stream.Length > 0)
            {
                // Обновление пользовательского интефейса,
                // указывающее на то, что звук проигрывается
                SetButtonStates(false, false, true);
                UserHelp.Text = "Воспроизведение записи";
                StatusImage.Source = speakerImage;

                // Проигрывает звук в новом потоке, что позволяет
                //пользовательскому интерфейсу обновляться.
                Thread soundThread = new Thread(new ThreadStart(playSound));
                soundThread.Start();
            }
        }

        /// <summary>
        /// Воспроизводит звук, используя SoundEffectInstance 
        /// это позволяет нам отслеживать состояние проигрывания.
        /// </summary>
        private void playSound()
        {
            // Воспроизводит звук, используя SoundEffectInstance 
            // что позволяет нам отслеживать состояние воспроизведения 
            // и обновлять пользовательский интерфейс в обработчике dt_Tick, когда воспроизведение завершитсяg.
            SoundEffect sound = new SoundEffect(stream.ToArray(), microphone.SampleRate, AudioChannels.Mono);
            soundInstance = sound.CreateInstance();
            soundIsPlaying = true;
            soundInstance.Play();
        }

        /// <summary>
        /// Вспомогательный метод для изменения свойства IsEnabled 
        /// для объектов ApplicationBarIconButtons (кнопок на панели приложения).
        /// </summary>
        /// <param name="recordEnabled">Новое состояние для кнопки record.</param>
        /// <param name="playEnabled">Новое состояние для  play .</param>
        /// <param name="stopEnabled">Новое состояние для stop .</param>
        private void SetButtonStates(bool recordEnabled, bool playEnabled, bool stopEnabled)
        {
            (ApplicationBar.Buttons[0] as ApplicationBarIconButton).IsEnabled = recordEnabled;
            (ApplicationBar.Buttons[1] as ApplicationBarIconButton).IsEnabled = playEnabled;
            (ApplicationBar.Buttons[2] as ApplicationBarIconButton).IsEnabled = stopEnabled;
        }
    }
}